/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.novabpm.identity.impl;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.ow2.novabpm.identity.IdentityServiceOp;
import org.ow2.novabpm.identity.IdentityServiceOpTest;
import org.ow2.novabpm.util.Misc;

/**
 * @author "Pierre Vigneras"
 * @date Nov 28, 2007
 */
public abstract class PersistentIdentityServiceTest extends
    IdentityServiceOpTest {
  public static final int TEST_NUMBER = 5;

  public static final Map<Class, File> files = new HashMap<Class, File>();

  public static final String STATIC_FILE_NAME_PROPERTY = "org.ow2.novabpm.identity.impl.persistent.staticFile";

  private static final File staticFile;
  static {
    final String s = System.getProperty(STATIC_FILE_NAME_PROPERTY);
    staticFile = (s == null) ? null : new File(s);
  }

  protected IdentityServiceOp newIdentityService() {
    try {
      Class<? extends PersistentIdentityServiceTest> clazz = this.getClass();
      File file = files.get(clazz);
      if (file == null) {
        file = File.createTempFile(clazz.getSimpleName(), ".tmp");
        files.put(clazz, file);
      }
      return newIdentityService(file, true);
    } catch (Exception ioe) {
      throw new RuntimeException(ioe);
    }
  }

  protected abstract PersistentIdentityService newIdentityService(
      final File file, boolean create) throws Exception;

  protected void testPersistence(final File file) throws Exception {
    System.out.println("Using file: " + file.getAbsolutePath());
    IdentityServiceOp identityServiceOp = newIdentityService(file, true);
    IdentityServiceOpTest.populate(identityServiceOp, MIN_GROUP_NB
        + Misc.RANDOM.nextInt(MAX_GROUP_NB - MIN_GROUP_NB));
    identityServiceOp.commit();
    identityServiceOp = newIdentityService(file, true);
    IdentityServiceOpTest.checkConsistency(identityServiceOp.getAllGroups(),
        identityServiceOp.getAllUsers());
  }

  public void testPersistence() throws Exception {
    if (staticFile != null) {
      if (staticFile.exists()) {
        IdentityServiceOp identityServiceOp = newIdentityService(staticFile,
            false);
        IdentityServiceOpTest.checkConsistency(
            identityServiceOp.getAllGroups(), identityServiceOp.getAllUsers());
      } else {
        testPersistence(staticFile);
      }
    } else {
      for (int i = 0; i < TEST_NUMBER; i++) {
        testPersistence(File.createTempFile(PersistentIdentityServiceTest.class
            .getSimpleName(), ".tmp"));
      }
    }
  }
}
